@@ -2,12 +2,16 @@ from __future__ import print_function
 
 import json
 import random
+import logging
 import bpy
 
 import kabaret
 from kabaret.app.ui.view import ViewMixin
 
 
+logger = logging.getLogger(__name__)
+
+
 #
 #   PROPERTIES
 #
@@ -39,26 +43,28 @@ class FlowFieldProperties(bpy.types.PropertyGroup):
         except KeyError:
             pass
         else:
-            print("Using cache", value, self.oid)
+            logger.debug("Using cache", value, self.oid)
             return value
 
-        print("GET VALUE FOR", self.oid)
+        logger.debug("GET VALUE FOR", self.oid)
         try:
             value = kabaret.session.cmds.Flow.get_value(self.oid)
         except Exception as err:
-            print("!!!! Error getting value for {}: {}".format(self.oid, err))
+            logger.debug("!!!! Error getting value for {}: {}".format(self.oid, err))
             value = "!!! ERROR: " + str(err)
 
         _FIELDS_VALUE_CACHE[self.oid] = value
-        return self._py_value_cache
+        return value
 
     def _set_value(self, value):
         kabaret.session.cmds.Flow.set_value(self.oid, value)
 
     def touch_value(self):
         global _FIELDS_VALUE_CACHE
-        del _FIELDS_VALUE_CACHE[self.oid]
-        return
+        try:
+            del _FIELDS_VALUE_CACHE[self.oid]
+        except KeyError:
+            pass
 
     def get_value(self):
         value = self._get_value()
@@ -135,13 +141,13 @@ class FlowPageProperties(bpy.types.PropertyGroup):
     def update_all_VIEW_3D_TOOLS(self):
         all_3d_areas = []
         for screen in bpy.data.screens:
-            # print('1', screen)
+            # logger.debug('1', screen)
             for area in screen.areas:
-                # print('2  ', area)
+                # logger.debug('2  ', area)
                 for space in area.spaces:
-                    # print('3  ', space)
+                    # logger.debug('3  ', space)
                     if space.type == "VIEW_3D":
-                        # print('+++AREA', area)
+                        # logger.debug('+++AREA', area)
                         all_3d_areas.append(area)
                         break
 
@@ -150,7 +156,7 @@ class FlowPageProperties(bpy.types.PropertyGroup):
                 # this does not redraw:
                 for region in area.regions:
                     if region.type == "TOOLS":
-                        # print(">>>>>>>>>>", region)
+                        # logger.debug(">>>>>>>>>>", region)
                         region.tag_redraw()
             else:
                 # but this does:
@@ -185,11 +191,17 @@ class FlowPageProperties(bpy.types.PropertyGroup):
         )
         group_name = set()
         for related in related_list:
-            name, relation_type, is_action, is_map, ui = related
+            try:
+                related_oid, name, relation_type, is_action, is_map, ui = related
+            except ValueError:
+                # Kabaret before version 2.2:
+                name, relation_type, is_action, is_map, ui = related
+                related_oid = oid + "/" + name
+
             field = bpy.context.window_manager.flow_page.fields.add()
             field.name = name
             # Flow
-            field.oid = oid + "/" + name
+            field.oid = related_oid
             field.relation_type = relation_type
             field.is_action = is_action
             field.is_map = is_map
@@ -216,10 +228,13 @@ class FlowPageProperties(bpy.types.PropertyGroup):
         update=on_current_oid_changed,
     )
     is_action: bpy.props.BoolProperty(
-        name="Flow Page Is Action", description="What name says",
+        name="Flow Page Is Action",
+        description="What name says",
     )
     action_needs_dialog: bpy.props.BoolProperty(name="Flow Page Is Action w/ dialog")
-    action_buttons: bpy.props.StringProperty(name="Flow Page Is Action w/ Buttons",)
+    action_buttons: bpy.props.StringProperty(
+        name="Flow Page Is Action w/ Buttons",
+    )
     open_states: bpy.props.CollectionProperty(
         type=OpenStateProperties,
         name="Page Groups Open States",
@@ -297,7 +312,8 @@ class KABARET_OT_RunAction(bpy.types.Operator):
     action_oid: bpy.props.StringProperty(name="parent oid")
     from_oid: bpy.props.StringProperty(name="flow_oid")
     page_is_dialog: bpy.props.BoolProperty(
-        name="the current page is the action dialog", default=False,
+        name="the current page is the action dialog",
+        default=False,
     )
     button: bpy.props.StringProperty(name="clicked button")
 
@@ -394,7 +410,8 @@ class KABARET_PT_flow_page(bpy.types.Panel):
         if current_oid.startswith(home_oid):
             path = "Home" + current_oid[len(home_oid) :]
         self.layout.menu(
-            KABARET_MT_NavigatorMenu.bl_idname, text=path,
+            KABARET_MT_NavigatorMenu.bl_idname,
+            text=path,
         )
 
         parent_oid = current_oid
@@ -456,19 +473,22 @@ class KABARET_PT_flow_page(bpy.types.Panel):
                 if not field.editable or relation_type == "Computed":
                     row.enabled = False
                 value_prop_name = "value"
-                if field.editor_type == "bool":
-                    value_prop_name = "bool_value"
-                elif field.editor_type == "int":
-                    value_prop_name = "int_value"
-                elif field.editor_type == "float":
-                    value_prop_name = "float_value"
+                if field.editor_type == "label":
+                    row.label(text=field.value)
+                else:
+                    if field.editor_type == "bool":
+                        value_prop_name = "bool_value"
+                    elif field.editor_type == "int":
+                        value_prop_name = "int_value"
+                    elif field.editor_type == "float":
+                        value_prop_name = "float_value"
 
-                row.prop(
-                    data=field,
-                    property=value_prop_name,
-                    text=field.label,
-                    translate=False,
-                )
+                    row.prop(
+                        data=field,
+                        property=value_prop_name,
+                        text=field.label,
+                        translate=False,
+                    )
 
             elif field.is_map:
                 icon = "ALIGN_JUSTIFY"
@@ -528,8 +548,14 @@ class KABARET_PT_flow_page(bpy.types.Panel):
                 goto_op.name = field.name
                 goto_op.parent_oid = parent_oid
 
+            elif relation_type in ("Connection", "Relative"):
+                lo.label(
+                    text="%s is not yet implemented for Blender Flow View"
+                    % relation_type
+                )
             else:
-                lo.label("???" + field.oid)
+
+                lo.label(text="???" + field.oid)
                 goto_op = self.layout.operator(
                     KABARET_OT_GotoRelated.bl_idname, text=field.label
                 )
@@ -588,16 +614,16 @@ class FlowView(ViewMixin):
         bpy.app.handlers.load_post.append(self._scene_load_handler)
 
     def receive_event(self, event, data):
-        print("FlowView receive_event", event, data)
+        logger.debug("FlowView receive_event", event, data)
         if event == "flow_touched":
             oid = data["oid"]
 
             if not oid.startswith(
                 bpy.context.window_manager.flow_page.current_oid + "/"
             ):
-                print("   touched oid not under current page oid")
-                print("      touched: " + oid)
-                print(
+                logger.debug("   touched oid not under current page oid")
+                logger.debug("      touched: " + oid)
+                logger.debug(
                     "      current: ",
                     bpy.context.window_manager.flow_page.current_oid + "/",
                 )
@@ -605,7 +631,7 @@ class FlowView(ViewMixin):
 
             for field in bpy.context.window_manager.flow_page.fields:
                 if field.oid == oid:
-                    print("     UPDATING VALUE FOR:", field)
+                    logger.debug("     UPDATING VALUE FOR:", field)
                     field.touch_value()
                     bpy.context.window_manager.flow_page.update_all_VIEW_3D_TOOLS()
 
